using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using OpenGL;

namespace PL02cs
{
    public partial class MainForm : Form
    {
        public MainForm()
        {
            InitializeComponent();

            openGLPanel1.Draw += new OpenGL.DrawHandler(openGLPanel1_Draw);
            openGLPanel1.Resize += new EventHandler(openGLPanel1_Resize);
        }

        void openGLPanel1_Resize(object sender, EventArgs e)
        {
            // setup
            GL.glViewport(0, 0, openGLPanel1.Width, openGLPanel1.Height);
            GL.glMatrixMode(GL.GL_PROJECTION);
            GL.glLoadIdentity();
            GL.glOrtho(
                //float.Parse(txtX0.Text) - 1, float.Parse(txtX1.Text) + 1,
                //float.Parse(txtY0.Text) - 1, float.Parse(txtY1.Text) + 1,
                -3, 3, -3, 3,
                -7.0,
                7.0
                );
            GL.glMatrixMode(GL.GL_MODELVIEW);
            GL.glLoadIdentity();
        }

        void openGLPanel1_Draw()
        {
            // clear window
            GL.glClearColor(0, 0, 0, 0);
            GL.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT);
            GL.glLoadIdentity();

            GL.glTranslatef(0.0f, 0.0f, -6.0f);

            // draw axis
            GL.glBegin(GL.GL_LINES);
            GL.glColor3f(1.0f, 0.0f, 0.0f);
            GL.glVertex2i(3, 0);
            GL.glVertex2i(-3, 0);
            GL.glColor3f(0.0f, 1.0f, 0.0f);
            GL.glVertex2i(0, 3);
            GL.glVertex2i(0, -3);
            GL.glEnd();

            if (calc != null)
            {
                // draw pin points
                GL.glPushAttrib(GL.GL_POINT_BIT);
                GL.glPointSize(5f);
                GL.glBegin(GL.GL_POINTS);
                GL.glColor3f(0.0f, 0.0f, 1.0f);
                GL.glVertex2f(calc.Center.X, calc.Center.Y);
                GL.glVertex2f(calc.P0.X, calc.P0.Y);
                GL.glVertex2f(calc.P1.X, calc.P1.Y);
                GL.glEnd();
                GL.glPopAttrib();
                GL.glPushAttrib(GL.GL_LINE_BIT);
                GL.glLineStipple(1, 0x0F0F);
                GL.glEnable(GL.GL_LINE_STIPPLE);
                GL.glBegin(GL.GL_LINES);
                GL.glColor3f(0.0f, 0.0f, 1.0f);
                GL.glVertex2f(calc.P0.X, calc.P0.Y);
                GL.glVertex2f(calc.P1.X, calc.P1.Y);
                GL.glEnd();
                GL.glPopAttrib();

                // draw circle
                GL.glBegin(GL.GL_LINE_LOOP);
                GL.glColor3f(1.0f, 1.0f, 1.0f);
                foreach (PointF p in calc.Points)
                    GL.glVertex2f(p.X, p.Y);
                GL.glEnd();
            }

            GL.glFlush();
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            txtX0.Text = PL02CalcEngine.DefaultP0.X.ToString();
            txtY0.Text = PL02CalcEngine.DefaultP0.Y.ToString();
            txtX1.Text = PL02CalcEngine.DefaultP1.X.ToString();
            txtY1.Text = PL02CalcEngine.DefaultP1.Y.ToString();
        }

        private PL02CalcEngine calc;

        private void btnCalc_Click(object sender, EventArgs e)
        {
            calc = new PL02CalcEngine(
                new PointF(float.Parse(txtX0.Text), float.Parse(txtY0.Text)),
                new PointF(float.Parse(txtX1.Text), float.Parse(txtY1.Text)),
                uint.Parse(txtNSegments.Text)
                );

            calc.Calculate();

            lblAlfa.Text = calc.Alfa.ToString();
            lblCenter.Text = calc.Center.ToString();
            lblDistance.Text = calc.Distance.ToString();

            lboxPoints.Items.Clear();
            foreach (PointF p in calc.Points)
                lboxPoints.Items.Add(p);

            openGLPanel1.Invalidate();
        }
    }
}