using System;
using System.Collections.Generic;
using System.Text;

using System.Drawing;

namespace PL02cs
{
    class PL02CalcEngine
    {
        public static readonly System.Drawing.PointF DefaultP0 = new System.Drawing.PointF(-2.0f, -2.0f);
        public static readonly System.Drawing.PointF DefaultP1 = new System.Drawing.PointF(2.0f, 2.0f);

        private PointF p0;

        public PointF P0
        {
            get { return p0; }
        }
        private PointF p1;

        public PointF P1
        {
            get { return p1; }
        }
        private uint nsegments;

        private bool IsCalculated = false;

        private PointF center;
        private float radius;
        private float distance;
        private float alfa;
        private IList<PointF> points;

        public PL02CalcEngine(PointF p0, PointF p1, uint nsegments)
        {
            this.p0 = p0;
            this.p1 = p1;
            this.nsegments = nsegments;
        }

        private float rad(float ang)
        {
            return ang * (float)Math.PI / 180.0f;
        }

        public void Calculate()
        {
            center = new PointF((p0.X + p1.X) / 2.0f, (p0.Y + p1.Y) / 2.0f);

            distance = (float)Math.Sqrt((double)(p1.X - p0.X) * (p1.X - p0.X) + (p1.Y - p0.Y) * (p1.Y - p0.Y));

            radius = distance / 2.0f;

            alfa = rad(360.0f) / (float)nsegments;

            points = new List<PointF>();
            double t = 0;
            for (int i = 0; i < nsegments; i++)
            {
                points.Add(new PointF(radius * (float)Math.Cos(t) + center.X, radius * (float)Math.Sin(t) + center.Y));
                t += alfa;
            }

            IsCalculated = true;
        }

        public PointF Center
        {
            get 
            {
                if (!IsCalculated)
                    throw new InvalidOperationException("You can only query output values after performing the calculation");
                return center;
            }
        }

        public float Distance
        {
            get
            {
                if (!IsCalculated)
                    throw new InvalidOperationException("You can only query output values after performing the calculation");
                return distance;
            }
        }

        public float Alfa
        {
            get
            {
                if (!IsCalculated)
                    throw new InvalidOperationException("You can only query output values after performing the calculation");
                return alfa;
            }
        }

        public IList<PointF> Points
        {
            get
            {
                if (!IsCalculated)
                    throw new InvalidOperationException("You can only query output values after performing the calculation");
                return new System.Collections.ObjectModel.ReadOnlyCollection<PointF>(points);
            }
        }
    }
}
